package com.engine.hrmelog.config;

import com.engine.hrmelog.entity.dto.LoggerContext;
import com.engine.hrmelog.mapper.ElogTableCheckerMapper;
import com.engine.hrmelog.util.db.IdGenerator;
import com.engine.hrmelog.util.db.MapperProxyFactory;


public class ELogTableChecker {
    private static final long version = 0;
    private ELogCache eLogCache = new ELogCache();

    private ElogTableCheckerMapper getTableCheckerMapper() {
        return MapperProxyFactory.getProxy(ElogTableCheckerMapper.class);
    }

    public <T> void check(LoggerContext<T> loggerContext) {
        synchronized (this){
            String module = loggerContext.getModuleName();
            String function = loggerContext.getFunctionName();
            String mainTable = String.format("%s_%slogs", module, function);
            Long v = eLogCache.getVersion(mainTable);
            boolean noTable = v == null;
            if (noTable) {
                this.initTable(mainTable);
                v = version;
            } else {
                this.checkVersion(mainTable, v);
            }
        }

    }

    private void checkVersion(String mainTable, Long v) {
        if (v == version) {
            return;
        }
    }


    private void initTable(String mainTable) {
        getTableCheckerMapper().recordVersion(IdGenerator.generate(), mainTable, version);
        getTableCheckerMapper().createMainTable(mainTable);
        getTableCheckerMapper().createDetailTable(mainTable + "_detail");
    }
}
